/* This file was generated by syncqt. */
#ifndef QT_QTSERVICEFRAMEWORK_VERSION_H
#define QT_QTSERVICEFRAMEWORK_VERSION_H

#define QTSERVICEFRAMEWORK_VERSION_STR "5.4.0"

#define QTSERVICEFRAMEWORK_VERSION 0x050400

#endif // QT_QTSERVICEFRAMEWORK_VERSION_H
