/* This file was generated by syncqt. */
#ifndef QT_QTSYSTEMINFO_VERSION_H
#define QT_QTSYSTEMINFO_VERSION_H

#define QTSYSTEMINFO_VERSION_STR "5.4.0"

#define QTSYSTEMINFO_VERSION 0x050400

#endif // QT_QTSYSTEMINFO_VERSION_H
