/* This file was generated by syncqt. */
#ifndef QT_QTPUBLISHSUBSCRIBE_VERSION_H
#define QT_QTPUBLISHSUBSCRIBE_VERSION_H

#define QTPUBLISHSUBSCRIBE_VERSION_STR "5.4.0"

#define QTPUBLISHSUBSCRIBE_VERSION 0x050400

#endif // QT_QTPUBLISHSUBSCRIBE_VERSION_H
